/*  D2vid - video conversion software for Cowon D2 portable media player
    Copyright (C) 2008 Wesley Stessens <wesley@ubuntu.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef AUDIOTRACKS_H
#define AUDIOTRACKS_H

#include <QMap>
#include <QString>

class QStringList;

/**
* \brief The AudioTrack class holds data for one audio track
* \sa AudioTrackList
*/
class AudioTrack {
public:
	/// Constructor
	AudioTrack();

	/// Destructor
	~AudioTrack();

	/**
	* \brief Retrieve the audio track ID
	* \return audio track ID
	*/
	int ID() const {return _id;};

	/**
	* \brief Retrieve the language code
	* \return audio track language code
	* \sa langName
	*
	* Retrieves the language code for this audio track.
	* Please note that the media might not provide a valid language code.
	* The media can provide any string as the language code.
	*/
	QString lang() const {return _lang;};

	/**
	* \brief Retrieve the full language name
	* \return full language name for this audio track
	* \sa langName
	*
	* Retrieve the full language name for this audio track.
	* Please note that the media might not provide a valid language name,
	* The media can provide any string as the language name.
	*/
	QString name() const {return _name;};

	/**
	* \brief Retrieve language code and language name in one string
	* \return string consisting of language code and language name
	* \sa lang, name, AudioTrackList::list
	*
	* This function combines the language code and the language name into
	* a logical and readable string. Please use this instead of directly calling
	* lang() or name() if you want a graphical representation of the audio track.
	*/
	QString langName() const;

	/**
	* \brief Set the audio track ID
	* \param id audio track ID
	*/
	void setID(int id) {_id = id;};

	/**
	* \brief Set the language code
	* \param lang language code
	*/
	void setLang(const QString &lang) {_lang = lang;};

	/**
	* \brief Set the full language name
	* \param id full language name
	*/
	void setName(const QString &name) {_name = name;};

private:
	int _id;
	QString _lang; // often language code
	QString _name; // often full language name
};

/**
* \brief The AudioTrackList class manages multiple AudioTracks
* \sa AudioTrack
*/
class AudioTrackList {
public:
	/// Constructor
	AudioTrackList();

	/// Destructor
	~AudioTrackList();

	/**
	* \brief Clear the audio track list (remove all audio tracks)
	*/
	void clear();

	/**
	* \brief Check if this audio track list is empty
	* \return \li \c true if this audio track list is empty
	*         \li \c false if there are audio tracks in this list
	*/
	bool isEmpty();

	/**
	* \brief Retrieve all audio tracks as a list of QStrings.
	* \return list of QStrings representing all audio tracks
	*
	* Retrieve all audio tracks as a list of QStrings. This function combines
	* the language codes and the language names into logical and readable strings.
	*/
	QStringList list();

	/**
	* \brief Add a new audio track
	* \param id ID of the new audio track
	*
	* Add a new audio track and give it an audio track ID.
	*/
	void newID(int id);

	/**
	* \brief Add language code to existing audio track
	* \param id ID of existing audio track
	* \param lang language code
	*/
	void newLang(int id, const QString &lang);

	/**
	* \brief Add full language name to existing audio track
	* \param id ID of existing audio track
	* \param name full language name
	*/
	void newName(int id, const QString &name);

private:
	typedef QMap<int, AudioTrack> AudioMap;
	AudioMap audio;
};

#endif
