/*  D2vid - video conversion software for Cowon D2 portable media player
    Copyright (C) 2008 Wesley Stessens <wesley@ubuntu.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mainwindow.h"
#include "dlgabout.h"
#include "mplayerprocess.h"
#include "core.h"
#include "media.h"
#include "common.h"
#include <QDirModel>
#include <QCompleter>
#include <QFileDialog>
#include <QFileInfo>

#include <QDebug>

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent) {
	//// Core initialization ////
	core = new Core(this);
	Common::init();

	//// Form initialization ////
	ui.setupUi(this);
	resize(minimumSize()); // Start small
	ui.labelUpdates->setOpenExternalLinks(true);

	//// Form connections ////
	connect(ui.actionQuit, SIGNAL(triggered()), qApp, SLOT(quit()));
	connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
	
	//// Input initialization ////
	ui.labelErrorVideo->hide();
	// Video auto-completion
	QCompleter *videoCompleter = new QCompleter(this);
	QDirModel *dirModelVideo = new QDirModel(videoCompleter);
	dirModelVideo->setSorting(QDir::Name|QDir::DirsLast);
	dirModelVideo->setFilter(QDir::AllDirs|QDir::AllEntries|QDir::NoDotAndDotDot);
	dirModelVideo->setNameFilters(Common::videoFilterList());
	videoCompleter->setModel(dirModelVideo);
	videoCompleter->setCaseSensitivity(Qt::CaseInsensitive);
	ui.lineVideo->setCompleter(videoCompleter);
	// Subtitle auto-completion
	QCompleter *subCompleter = new QCompleter(this);
	QDirModel *dirModelSub = new QDirModel(subCompleter);
	dirModelSub->setSorting(QDir::Name|QDir::DirsLast);
	dirModelSub->setFilter(QDir::AllDirs|QDir::AllEntries|QDir::NoDotAndDotDot);
	dirModelSub->setNameFilters(Common::subFilterList());
	subCompleter->setModel(dirModelSub);
	subCompleter->setCaseSensitivity(Qt::CaseInsensitive);
	ui.lineSub->setCompleter(subCompleter);
	// External audio auto-completion
	QCompleter *audioCompleter = new QCompleter(this);
	QDirModel *dirModelAudio = new QDirModel(audioCompleter);
	dirModelAudio->setSorting(QDir::Name|QDir::DirsLast);
	dirModelAudio->setFilter(QDir::AllDirs|QDir::AllEntries|QDir::NoDotAndDotDot);
	dirModelAudio->setNameFilters(Common::audioFilterList());
	audioCompleter->setModel(dirModelAudio);
	audioCompleter->setCaseSensitivity(Qt::CaseInsensitive);
	ui.lineAudioExt->setCompleter(audioCompleter);
	setAudioExt(false);
	enableAudio(false);
	enableSubs(false);

	//// Input connections ////
	connect(ui.checkSub, SIGNAL(toggled(bool)), this, SLOT(enableSubs(bool)));
	connect(ui.checkAudio, SIGNAL(toggled(bool)), this, SLOT(enableAudio(bool)));
	connect(ui.radioAudioExt, SIGNAL(toggled(bool)), this, SLOT(setAudioExt(bool)));
	connect(ui.pushVideo, SIGNAL(clicked()), this, SLOT(browseVideo()));
	connect(ui.pushSub, SIGNAL(clicked()), this, SLOT(browseSub()));
	connect(ui.pushAudioExt, SIGNAL(clicked()), this, SLOT(browseAudio()));
	connect(ui.lineVideo, SIGNAL(textChanged(const QString&)), this, SLOT(newVideo(const QString&)));

	//// Preview initialization ////
	enablePreview(false);
	ui.sliderMediaPos->setTracking(false);

	//// Preview connections ////
	connect(ui.pushPlay, SIGNAL(clicked()), this, SLOT(playPreview()));
	connect(ui.push1mL, SIGNAL(clicked()), this, SLOT(seekMedia1mL()));
	connect(ui.push1mR, SIGNAL(clicked()), this, SLOT(seekMedia1mR()));
	connect(ui.push10sL, SIGNAL(clicked()), this, SLOT(seekMedia10sL()));
	connect(ui.push10sR, SIGNAL(clicked()), this, SLOT(seekMedia10sR()));
	connect(ui.sliderMediaPos, SIGNAL(sliderValueChanged(int)), core, SLOT(seekMediaAbs(int)));

	//// Encoding initialization ////
	calcBitrate();

	//// Encoding connections ////
	connect(ui.spinVbr, SIGNAL(valueChanged(int)), this, SLOT(calcBitrate()));
	connect(ui.spinAbr, SIGNAL(valueChanged(int)), this, SLOT(calcBitrate()));

	//// Subtitles connections ////
	// The QSlider-QSpinbox connections are created in Qt Designer
}

MainWindow::~MainWindow() {
}

void MainWindow::analyseMedia() {
	ui.listAudioInt->clear();
	if (Media::ok()) {
		ui.labelErrorVideo->hide();
		if (!Media::audioList.isEmpty()) {
			ui.listAudioInt->addItems(Media::audioList.list());
			ui.listAudioInt->setCurrentRow(0);
			if (ui.radioAudioInt->isChecked()) ui.checkAudio->setChecked(true);
		}
		enablePreview(true);
	} else {
		ui.labelErrorVideo->setText(Common::error("File analysis failed. Could not process the input file with mplayer."));
		ui.labelErrorVideo->show();
		enablePreview(false);
	}
}

void MainWindow::stopMedia() {
	if (core->proc()->type() == MPlayerProcess::Preview) {
		ui.preview->unloadMedia();
		ui.pushPlay->setText("Pl&ay");
		ui.labelPos->setText("00:00:00");
		ui.sliderMediaPos->setValue(0);
		ui.sliderMediaPos->setEnabled(false);
		Media::setActive(false);
	}
}

void MainWindow::showAbout() {
	DlgAbout *about = new DlgAbout(this); // dialog deletes itself
	about->show();
}

// Input

void MainWindow::enableSubs(bool enable) {
	ui.lineSub->setEnabled(enable);
	ui.pushSub->setEnabled(enable);
}

void MainWindow::enableAudio(bool enable) {
	ui.lineAudioExt->setEnabled(enable);
	ui.pushAudioExt->setEnabled(enable);
	ui.listAudioInt->setEnabled(enable);
	ui.radioAudioInt->setEnabled(enable);
	ui.radioAudioExt->setEnabled(enable);
}

void MainWindow::setAudioExt(bool external) {
	ui.lineAudioExt->setVisible(external);
	ui.pushAudioExt->setVisible(external);
	ui.listAudioInt->setVisible(!external);
}

void MainWindow::browseVideo() {
	QString showDir(ui.lineVideo->text());
	if (showDir.isEmpty() || !QFile::exists(showDir)) showDir = prevDirectory;
	if (showDir.isEmpty()) showDir = QDir::homePath();
	QString fileName = QFileDialog::getOpenFileName(this, "Select video file", showDir,
	  QString("Video files (%1);;All files (*.*)").arg(Common::videoFilterString()));
	if (!fileName.isEmpty()) {
		ui.lineVideo->setText(fileName);
		prevDirectory = QDir(QFileInfo(fileName).dir()).absolutePath();
	}
}

void MainWindow::browseSub() {
	QString showDir(ui.lineSub->text());
	if (showDir.isEmpty() || !QFile::exists(showDir)) showDir = prevDirectory;
	if (showDir.isEmpty()) showDir = QDir::homePath();
	QString fileName = QFileDialog::getOpenFileName(this, "Select subtitles", showDir,
	  QString("Subtitles (%1);;All files (*.*)").arg(Common::subFilterString()));
	if (!fileName.isEmpty()) {
		ui.lineSub->setText(fileName);
		prevDirectory = QDir(QFileInfo(fileName).dir()).absolutePath();
	}
}

void MainWindow::browseAudio() {
	QString showDir(ui.lineAudioExt->text());
	if (showDir.isEmpty() || !QFile::exists(showDir)) showDir = prevDirectory;
	if (showDir.isEmpty()) showDir = QDir::homePath();
	QString fileName = QFileDialog::getOpenFileName(this, "Select audio file", showDir,
	  QString("Audio files (%1);;All files (*.*)").arg(Common::audioFilterString()));
	if (!fileName.isEmpty()) {
		ui.lineAudioExt->setText(fileName);
		prevDirectory = QDir(QFileInfo(fileName).dir()).absolutePath();
	}
}

void MainWindow::newVideo(const QString &fileName) {
	Media::reset();
	ui.listAudioInt->clear();

	QFileInfo fileInfo(fileName);
	if (fileInfo.exists() && fileInfo.isFile()) {
		ui.listAudioInt->addItem("Please wait...");
		core->createAnalysisProcess(fileName);
		connect(core->proc(), SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(analyseMedia()));
		core->proc()->runProcess();
	}
}

// Preview

void MainWindow::enablePreview(bool enable) {
	ui.push1mL->setEnabled(enable);
	ui.push1mR->setEnabled(enable);
	ui.push10sL->setEnabled(enable);
	ui.push10sR->setEnabled(enable);
	ui.pushPlay->setEnabled(enable);
}

void MainWindow::setPaused(bool pause) {
	if (pause) {
		ui.pushPlay->setText("Pl&ay");
	} else {
		ui.pushPlay->setText("P&ause");
	}
}

void MainWindow::playPreview() {
	if (Media::active()) {
		core->proc()->write("pause\n");
	} else {
		core->createPreviewProcess(ui.lineVideo->text(), (int)ui.preview->widget()->winId());
		connect(core->proc(), SIGNAL(playbackPaused(bool)), this, SLOT(setPaused(bool)));
		connect(core->proc(), SIGNAL(gotResolution(int, int)), ui.preview->widget(),
		  SLOT(applyResolution(int, int)));
		connect(core->proc(), SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(stopMedia()));
		connect(core->proc(), SIGNAL(updatePosition(int)), this, SLOT(setPosition(int)));
		connect(core->proc(), SIGNAL(gotLength(int)), this, SLOT(setLength(int)));
		core->proc()->runProcess();
		ui.pushPlay->setText("P&ause");
	}
}

void MainWindow::seekMedia1mL() {
	core->seekMediaRel(-60);
}

void MainWindow::seekMedia1mR() {
	core->seekMediaRel(60);
}

void MainWindow::seekMedia10sL() {
	core->seekMediaRel(-10);
}

void MainWindow::seekMedia10sR() {
	core->seekMediaRel(10);
}

void MainWindow::setPosition(int sec) {
	// Small sanity check
	if (sec >= 0) {
		ui.labelPos->setText(Common::readablePosition(sec));
		if (ui.sliderMediaPos->isEnabled()) ui.sliderMediaPos->setSliderValue(sec);
	}
}

void MainWindow::setLength(int sec) {
	ui.sliderMediaPos->setMaximum(sec);
	ui.sliderMediaPos->setEnabled(true);
}

// Encoding

void MainWindow::calcBitrate() {
	QString bitrate = QString::number(ui.spinVbr->value() + ui.spinAbr->value());
	ui.labelEstSize->setText("Total bitrate: <span style='font-weight: bold'>"+ bitrate +" kbps</span><br />\nEstimated filesize: Unknown (no input file specified)");
}
