/*  D2vid - video conversion software for Cowon D2 portable media player
    Copyright (C) 2008 Wesley Stessens <wesley@ubuntu.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mplayerprocess.h"
#include "media.h"

#include <QDebug>

MPlayerProcess::MPlayerProcess(ProcType type, QObject *parent) : QProcess(parent) {
	_type = type;
	setProcessChannelMode(QProcess::MergedChannels);
	connect(this, SIGNAL(readyReadStandardOutput()), this, SLOT(processBytes()));
	connect(this, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(end()));
}

MPlayerProcess::~MPlayerProcess() {
}

void MPlayerProcess::processBytes() {
	QStringList lines = QString::fromLocal8Bit(readAllStandardOutput()).split("\n");
	foreach (QString line, lines) {
		processLine(line);
	}
}

void MPlayerProcess::end() {
	Media::setActive(false);
}

void MPlayerProcess::runProcess() {
	start(_procRunCommand);
}

// these regexes were partly lovingly stolen from smplayer - great movie player btw!
static QRegExp rx_av("^[AV]: *([0-9,:.-]+)");
static QRegExp rx_generic("^(.*)=(.*)");
static QRegExp rx_audio("^ID_AID_(\\d+)_(LANG|NAME)=(.*)");
static QRegExp rx_playing("^Starting playback...$");
static QRegExp rx_pause("^  =====  PAUSE  =====");
static QRegExp rx_resolution("^VO: \\[(.*)\\] \\d+x\\d+ => (\\d+)x(\\d+)");

void MPlayerProcess::processLine(QString line) {
	if (rx_av.indexIn(line) > -1) {
		if (Media::state() == Media::Paused) emit playbackPaused(false);
		int pos = int(rx_av.cap(1).toDouble());
		// Only emit a new position when a full second has changed
		if (prevPosition != pos) {
			emit updatePosition(pos);
			prevPosition = pos;
		}
	} else {
		// Windows has \r\n lines instead of \n lines, so we also have to chop \r from the line
		if (line.endsWith('\r')) line.chop(1);
		qDebug() << "MPLAYER LINE:" << line;
		if (rx_audio.indexIn(line) > -1) {
			processAudioLine(rx_audio.cap(1).toInt(), rx_audio.cap(3), rx_audio.cap(2));
		} else if (rx_playing.indexIn(line) > -1) {
			Media::setOk(true);
			Media::setActive(true);
			Media::setState(Media::Stopped);
			emit playbackStarted();
		} else if (rx_pause.indexIn(line) > -1) {
			Media::setState(Media::Paused);
			emit playbackPaused(true);
		} else if (rx_resolution.indexIn(line) > -1) {
			int w = rx_resolution.cap(2).toInt();
			int h = rx_resolution.cap(3).toInt();
			emit gotResolution(w, h);
		} else if (rx_generic.indexIn(line) > -1) {
			QString id = rx_generic.cap(1);
			QString val = rx_generic.cap(2);
			if (id == "ID_AUDIO_ID") processAudioLine(val.toInt());
			else if (id == "ID_LENGTH") emit gotLength(int(val.toDouble()));
		}
	}
}

void MPlayerProcess::processAudioLine(int id, QString desc, QString type) {
	Media::audioList.newID(id);
	if (!desc.isEmpty()) {
		if (type == "LANG") Media::audioList.newLang(id, desc);
		else if (type == "NAME") Media::audioList.newName(id, desc);
	}
}
